import asyncio
import concurrent.futures
from functools import lru_cache
import re
from typing import List, Optional, Dict, Any
import torch
import redis
from text_cleaner_for_py.cleaner import clean_text

class PerformanceTextCleaner:
    def __init__(self, max_workers: int = 4, cache_size: int = 1000):
        self.max_workers = max_workers
        self._cache = {}
        self.cache_size = cache_size
        self._redis_client = None

    def clean_texts_parallel(self, texts: List[str]) -> List[str]:
        """Processa múltiplos textos em paralelo."""
        with concurrent.futures.ThreadPoolExecutor(max_workers=self.max_workers) as executor:
            return list(executor.map(clean_text, texts))

    def clean_large_text(self, text: str, chunk_size: int = 1000) -> str:
        """Processa um texto grande dividindo em chunks."""
        chunks = [text[i:i + chunk_size] for i in range(0, len(text), chunk_size)]
        cleaned_chunks = self.clean_texts_parallel(chunks)
        return " ".join(cleaned_chunks)

    @lru_cache(maxsize=1000)
    def clean_text_cached(self, text: str) -> str:
        """Limpa o texto usando cache local."""
        return clean_text(text)

    def remove_ocr_noise(self, text: str) -> str:
        """Remove ruído comum em textos de OCR."""
        # Substitui números por letras comuns em OCR
        ocr_replacements = {
            '0': 'o', '1': 'i', '3': 'e', '4': 'a', '5': 's',
            '7': 't', '8': 'b', '9': 'g'
        }
        for num, letter in ocr_replacements.items():
            text = text.replace(num, letter)
        return text

    def normalize_measurements(self, text: str) -> str:
        """Normaliza unidades de medida no texto."""
        # Padrões comuns de medidas
        patterns = {
            r'(\d+(?:\.\d+)?)\s*kg': r'\1 quilogramas',
            r'(\d+(?:\.\d+)?)\s*m': r'\1 metros',
            r'(\d+(?:\.\d+)?)\s*cm': r'\1 centímetros',
            r'(\d+(?:\.\d+)?)\s*l': r'\1 litros',
            r'(\d+(?:\.\d+)?)\s*ml': r'\1 mililitros'
        }
        
        for pattern, replacement in patterns.items():
            text = re.sub(pattern, replacement, text, flags=re.IGNORECASE)
        return text

    def remove_duplicates(self, text: str) -> str:
        """Remove conteúdo duplicado no texto, considerando frases e sentenças."""
        # Divide por pontuação de fim de frase
        import string
        import re
        # Divide por ! ? .
        sentences = re.split(r'[.!?]', text)
        unique_sentences = []
        seen = set()
        for sentence in sentences:
            sentence = sentence.strip()
            if sentence and sentence not in seen:
                seen.add(sentence)
                unique_sentences.append(sentence)
        return '. '.join(unique_sentences) + ('.' if unique_sentences else '')

    def normalize_proper_names(self, text: str) -> str:
        """Normaliza nomes próprios no texto."""
        def capitalize_name(name):
            words = name.split()
            result = []
            for i, word in enumerate(words):
                if i == 0 or word.lower() not in ['da', 'de', 'do', 'das', 'dos', 'e']:
                    result.append(word.capitalize())
                else:
                    result.append(word.lower())
            return ' '.join(result)
        
        # Padrão para nomes próprios
        pattern = r'\b([a-zà-ú]+(?:\s+(?:da|de|do|das|dos|e)\s+[a-zà-ú]+)*)\b'
        
        def replace_name(match):
            return capitalize_name(match.group(1))
        
        return re.sub(pattern, replace_name, text, flags=re.IGNORECASE)

    async def clean_texts_async(self, texts: List[str]) -> List[str]:
        """Processa textos de forma assíncrona."""
        async def clean_single_text(text):
            return clean_text(text)
        
        tasks = [clean_single_text(text) for text in texts]
        return await asyncio.gather(*tasks)

    def is_gpu_available(self) -> bool:
        """Verifica se GPU está disponível."""
        return torch.cuda.is_available()

    def clean_text_gpu(self, text: str) -> str:
        """Processa texto usando GPU se disponível."""
        if not self.is_gpu_available():
            return clean_text(text)
        
        # Implementação básica de processamento GPU
        # Em um caso real, você usaria operações vetorizadas do PyTorch
        return clean_text(text)

    def is_redis_available(self) -> bool:
        """Verifica se Redis está disponível."""
        try:
            if self._redis_client is None:
                self._redis_client = redis.Redis(host='localhost', port=6379, db=0)
            self._redis_client.ping()
            return True
        except:
            return False

    def clean_text_distributed_cache(self, text: str) -> str:
        """Limpa texto usando cache distribuído (Redis)."""
        if not self.is_redis_available():
            return clean_text(text)
        
        cache_key = f"text_cleaner:{hash(text)}"
        cached_result = self._redis_client.get(cache_key)
        
        if cached_result:
            return cached_result.decode('utf-8')
        
        cleaned_text = clean_text(text)
        self._redis_client.set(cache_key, cleaned_text)
        return cleaned_text

    def clean_text_with_options(self, text: str, options: Dict[str, bool]) -> str:
        """Limpa texto com opções específicas."""
        result = text
        
        if options.get('remove_emojis', False):
            result = re.sub(r'[^\w\s-]', '', result)
        
        if options.get('remove_urls', False):
            result = re.sub(r'https?://\S+', '', result)
        
        return clean_text(result) 


from typing import List, Dict, Optional
from spellchecker import SpellChecker
import re

class SpellCheckerCleaner:
    def __init__(self, language: str = 'pt'):
        """
        Inicializa o corretor ortográfico.
        
        Args:
            language (str): Idioma para correção ('pt' para português, 'en' para inglês)
        """
        self.spell = SpellChecker(language=language)
        self.language = language
        self.abbreviations = {
            'vc': 'você',
            'tb': 'também',
            'pq': 'porque',
            'q': 'que',
            'td': 'tudo',
            'vlw': 'valeu',
            'blz': 'beleza',
            'tbm': 'também',
            'pfv': 'por favor',
            'pf': 'por favor',
            'obg': 'obrigado',
            'obgd': 'obrigado',
            'tks': 'thanks',
            'thx': 'thanks',
            'pls': 'please',
            'ty': 'thank you',
        }
        
    def check_text(self, text: str) -> Dict[str, List[str]]:
        """
        Verifica erros ortográficos no texto.
        
        Args:
            text (str): Texto a ser verificado
            
        Returns:
            Dict[str, List[str]]: Dicionário com palavras incorretas e sugestões
        """
        # Remove pontuação e divide em palavras
        words = re.findall(r'\b\w+\b', text.lower())
        
        # Encontra palavras incorretas
        misspelled = self.spell.unknown(words)
        
        # Cria dicionário com palavras incorretas e suas sugestões
        result = {}
        for word in misspelled:
            result[word] = list(self.spell.candidates(word))
            
        return result
    
    def correct_text(self, text: str) -> str:
        """
        Corrige erros ortográficos no texto.
        
        Args:
            text (str): Texto a ser corrigido
            
        Returns:
            str: Texto corrigido
        """
        # Divide o texto em palavras e pontuação
        words = re.findall(r'\b\w+\b|[^\w\s]|\s+', text)
        corrected_words = []
        for word in words:
            if re.match(r'\b\w+\b', word):  # Se for uma palavra
                lower_word = word.lower()
                if lower_word in self.abbreviations:
                    correction = self.abbreviations[lower_word]
                    if word[0].isupper():
                        correction = correction.capitalize()
                    corrected_words.append(correction)
                elif lower_word in self.spell:
                    corrected_words.append(word)
                else:
                    correction = self.spell.correction(word)
                    if word[0].isupper():
                        correction = correction.capitalize()
                    corrected_words.append(correction)
            else:
                corrected_words.append(word)
        return ''.join(corrected_words)
    
    def get_suggestions(self, word: str) -> List[str]:
        """
        Obtém sugestões de correção para uma palavra.
        
        Args:
            word (str): Palavra para obter sugestões
            
        Returns:
            List[str]: Lista de sugestões de correção
        """
        return list(self.spell.candidates(word)) 


import pytest
from text_cleaner_for_py.spell_checker import SpellCheckerCleaner

@pytest.fixture
def spell_checker():
    return SpellCheckerCleaner(language='pt')

def test_check_text(spell_checker):
    text = "Olá mundu! Como vai vc?"
    result = spell_checker.check_text(text)
    assert 'mundu' in result
    assert 'vc' in result
    assert len(result) == 2

def test_correct_text(spell_checker):
    text = "Olá mundu! Como vai vc?"
    corrected = spell_checker.correct_text(text)
    assert "mundo" in corrected.lower()
    assert "você" in corrected.lower()

def test_get_suggestions(spell_checker):
    word = "mundu"
    suggestions = spell_checker.get_suggestions(word)
    assert "mundo" in suggestions

def test_preserve_capitalization(spell_checker):
    text = "Olá Mundu!"
    corrected = spell_checker.correct_text(text)
    assert corrected == "Olá Mundo!"

def test_ignore_punctuation(spell_checker):
    text = "Olá, mundu! Como vai?"
    corrected = spell_checker.correct_text(text)
    assert "," in corrected
    assert "!" in corrected
    assert "?" in corrected

def test_empty_text(spell_checker):
    text = ""
    result = spell_checker.check_text(text)
    assert result == {}
    corrected = spell_checker.correct_text(text)
    assert corrected == ""

def test_no_errors(spell_checker):
    text = "Olá mundo! Como vai você?"
    result = spell_checker.check_text(text)
    assert result == {}
    corrected = spell_checker.correct_text(text)
    assert corrected == text 


import pytest
import os
from pathlib import Path
from text_cleaner_for_py.document_processor import DocumentProcessor

@pytest.fixture
def processor():
    return DocumentProcessor()

@pytest.fixture
def test_files_dir(tmp_path):
    # Criar diretório temporário para testes
    return tmp_path

@pytest.fixture
def sample_txt_file(test_files_dir):
    file_path = test_files_dir / "test.txt"
    content = "Este é um arquivo de teste.\nSegunda linha."
    file_path.write_text(content, encoding='utf-8')
    return str(file_path)

@pytest.fixture
def sample_docx_file(test_files_dir):
    # TODO: Criar arquivo DOCX de teste
    return None

@pytest.fixture
def sample_pdf_file(test_files_dir):
    # TODO: Criar arquivo PDF de teste
    return None

def test_read_txt_file(processor, sample_txt_file):
    content = processor.read_document(sample_txt_file)
    assert "Este é um arquivo de teste" in content
    assert "Segunda linha" in content

def test_file_not_found(processor):
    with pytest.raises(FileNotFoundError):
        processor.read_document("arquivo_inexistente.txt")

def test_unsupported_format(processor, test_files_dir):
    file_path = test_files_dir / "test.xyz"
    file_path.write_text("teste")
    with pytest.raises(ValueError):
        processor.read_document(str(file_path))

def test_extract_metadata_txt(processor, sample_txt_file):
    metadata = processor.extract_metadata(sample_txt_file)
    assert metadata['filename'] == "test.txt"
    assert metadata['extension'] == ".txt"
    assert metadata['size'] > 0
    assert 'created' in metadata
    assert 'modified' in metadata

@pytest.mark.skipif(True, reason="Necessário implementar criação de arquivos de teste")
def test_read_docx_file(processor, sample_docx_file):
    content = processor.read_document(sample_docx_file)
    assert content

@pytest.mark.skipif(True, reason="Necessário implementar criação de arquivos de teste")
def test_read_pdf_file(processor, sample_pdf_file):
    content = processor.read_document(sample_pdf_file)
    assert content

@pytest.mark.skipif(True, reason="Necessário implementar criação de arquivos de teste")
def test_extract_tables(processor, sample_docx_file):
    tables = processor.extract_tables(sample_docx_file)
    assert isinstance(tables, list)

@pytest.mark.skipif(True, reason="Necessário implementar criação de arquivos de teste")
def test_extract_images(processor, sample_docx_file):
    images = processor.extract_images(sample_docx_file)
    assert isinstance(images, list) 


import re
import unicodedata
from typing import Literal, Union

from bs4 import BeautifulSoup
from .exceptions import ValidationError, UnsupportedFormatError


# 🔡 Remover acentos e normalizar texto
def remove_accents(text: str) -> str:
    """
    Remove acentos do texto.
    
    Args:
        text: Texto de entrada com acentos
        
    Returns:
        Texto sem acentos
        
    Examples:
        >>> remove_accents("Olá, você está bem?")
        'Ola, voce esta bem?'
    """
    if not isinstance(text, str):
        raise ValidationError("text", text, "str")
    
    nfkd_form = unicodedata.normalize('NFKD', text)
    return u"".join([c for c in nfkd_form if not unicodedata.combining(c)])


# ✂ Remover caracteres especiais
def remove_special_characters(text: str) -> str:
    """
    Remove caracteres especiais do texto.
    
    Args:
        text: Texto de entrada com caracteres especiais
        
    Returns:
        Texto sem caracteres especiais
        
    Examples:
        >>> remove_special_characters("Olá! @#$%")
        'Olá '
    """
    if not isinstance(text, str):
        raise ValidationError("text", text, "str")
    
    return re.sub(r'[^\w\s]', '', text)


# 🔄 Converter para snake_case
def to_snake_case(text: str) -> str:
    """
    Converte texto para snake_case.
    
    Args:
        text: Texto de entrada para conversão
        
    Returns:
        Texto convertido para snake_case
        
    Examples:
        >>> to_snake_case("Texto Exemplo Aqui!")
        'texto_exemplo_aqui'
    """
    if not isinstance(text, str):
        raise ValidationError("text", text, "str")
    
    text = remove_accents(text)
    text = re.sub(r'\s+', '_', text.strip().lower())
    return re.sub(r'[^a-z0-9_]', '', text)


# 🔄 Converter para camelCase
def to_camel_case(text: str) -> str:
    """
    Converte texto para camelCase.
    
    Args:
        text: Texto de entrada para conversão
        
    Returns:
        Texto convertido para camelCase
        
    Examples:
        >>> to_camel_case("Texto Exemplo Aqui!")
        'textoExemploAqui'
    """
    if not isinstance(text, str):
        raise ValidationError("text", text, "str")
    
    text = to_snake_case(text)
    components = text.split('_')
    return components[0] + ''.join(x.title() for x in components[1:])


# 🔄 Converter para PascalCase
def to_pascal_case(text: str) -> str:
    """
    Converte texto para PascalCase.
    
    Args:
        text: Texto de entrada para conversão
        
    Returns:
        Texto convertido para PascalCase
        
    Examples:
        >>> to_pascal_case("Texto Exemplo Aqui!")
        'TextoExemploAqui'
    """
    if not isinstance(text, str):
        raise ValidationError("text", text, "str")
    
    return ''.join(word.capitalize() for word in to_snake_case(text).split('_'))


# 🌐 Remover HTML
def remove_html(text: str) -> str:
    """
    Remove tags HTML do texto.
    
    Args:
        text: Texto HTML de entrada
        
    Returns:
        Texto limpo sem tags HTML
        
    Examples:
        >>> remove_html("<div><h1>Título</h1><p>Parágrafo</p></div>")
        'Título Parágrafo'
    """
    if not isinstance(text, str):
        raise ValidationError("text", text, "str")
    
    soup = BeautifulSoup(text, 'html.parser')
    return soup.get_text(separator=' ', strip=True)


# 🔍 Remover múltiplos espaços
def remove_extra_spaces(text: str) -> str:
    """
    Remove espaços extras do texto.
    
    Args:
        text: Texto com espaços extras
        
    Returns:
        Texto com espaços normalizados
        
    Examples:
        >>> remove_extra_spaces("Texto    com   espaços   extras.")
        'Texto com espaços extras.'
    """
    if not isinstance(text, str):
        raise ValidationError("text", text, "str")
    
    return re.sub(r'\s+', ' ', text).strip()


# 🧹 Limpeza completa do texto
def clean_text(text: str, case: Literal['lower', 'upper', 'title', 'snake', 'camel', 'pascal'] = 'lower') -> str:
    """
    Limpeza completa do texto:
    - Remove HTML
    - Remove acentos
    - Remove caracteres especiais
    - Normaliza espaços
    - Converte o texto para o formato especificado
    
    Args:
        text: Texto de entrada para limpeza
        case: Formato de saída desejado ('lower', 'upper', 'title', 'snake', 'camel', 'pascal')
        
    Returns:
        Texto limpo no formato especificado
        
    Raises:
        ValidationError: Se o parâmetro 'text' não for uma string
        UnsupportedFormatError: Se o parâmetro 'case' for inválido
        
    Examples:
        >>> clean_text("<h1>Olá, Mundo!</h1>", case="lower")
        'ola mundo'
        >>> clean_text("Texto Exemplo", case="snake")
        'texto_exemplo'
    """
    if not isinstance(text, str):
        raise ValidationError("text", text, "str")
    
    if case not in ('lower', 'upper', 'title', 'snake', 'camel', 'pascal'):
        raise UnsupportedFormatError(case, ['lower', 'upper', 'title', 'snake', 'camel', 'pascal'])
    
    text = remove_html(text)
    text = remove_accents(text)
    text = remove_special_characters(text)
    text = remove_extra_spaces(text)

    if case == 'lower':
        return text.lower()
    elif case == 'upper':
        return text.upper()
    elif case == 'title':
        return text.title()
    elif case == 'snake':
        return to_snake_case(text)
    elif case == 'camel':
        return to_camel_case(text)
    elif case == 'pascal':
        return to_pascal_case(text)
    else:
        # Este ponto nunca deve ser alcançado devido à validação acima
        raise RuntimeError("Erro interno: formato de case inválido")






"""
Sistema de logging estruturado para o text_cleaner_for_py.

Este módulo fornece um sistema de logging configurável e estruturado
para monitorar o funcionamento da biblioteca.
"""

import logging
import logging.handlers
import sys
from pathlib import Path
from typing import Optional

from .config import config
from .exceptions import ConfigurationError


class ColoredFormatter(logging.Formatter):
    """Formatador de log com cores para terminal."""
    
    # Cores ANSI
    COLORS = {
        'DEBUG': '\033[36m',      # Cyan
        'INFO': '\033[32m',       # Green
        'WARNING': '\033[33m',    # Yellow
        'ERROR': '\033[31m',      # Red
        'CRITICAL': '\033[35m',   # Magenta
        'RESET': '\033[0m'        # Reset
    }
    
    def format(self, record: logging.LogRecord) -> str:
        """Formata o registro de log com cores."""
        # Adiciona cor ao nível do log
        level_color = self.COLORS.get(record.levelname, '')
        reset_color = self.COLORS['RESET']
        
        # Formata a mensagem
        formatted = super().format(record)
        
        # Adiciona cores se estiver no terminal
        if hasattr(sys.stderr, 'isatty') and sys.stderr.isatty():
            formatted = f"{level_color}{formatted}{reset_color}"
        
        return formatted


class StructuredFormatter(logging.Formatter):
    """Formatador de log estruturado para arquivos."""
    
    def format(self, record: logging.LogRecord) -> str:
        """Formata o registro de log de forma estruturada."""
        # Informações básicas
        log_entry = {
            'timestamp': self.formatTime(record),
            'level': record.levelname,
            'logger': record.name,
            'message': record.getMessage(),
            'module': record.module,
            'function': record.funcName,
            'line': record.lineno
        }
        
        # Adiciona exceção se houver
        if record.exc_info:
            log_entry['exception'] = self.formatException(record.exc_info)
        
        # Adiciona campos extras se houver
        if hasattr(record, 'extra_fields'):
            log_entry.update(record.extra_fields)
        
        # Formata como JSON-like (simplificado)
        formatted_parts = []
        for key, value in log_entry.items():
            if isinstance(value, str):
                formatted_parts.append(f"{key}='{value}'")
            else:
                formatted_parts.append(f"{key}={value}")
        
        return " | ".join(formatted_parts)


class TextCleanerLogger:
    """Logger principal para o text_cleaner_for_py."""
    
    def __init__(self, name: str = "text_cleaner_for_py") -> None:
        """
        Inicializa o logger.
        
        Args:
            name: Nome do logger
        """
        self.name = name
        self.logger = logging.getLogger(name)
        self._setup_logger()
    
    def _setup_logger(self) -> None:
        """Configura o logger com handlers e formatadores."""
        # Remove handlers existentes
        for handler in self.logger.handlers[:]:
            self.logger.removeHandler(handler)
        
        # Configura o nível do logger
        log_config = config.get_logging_config()
        self.logger.setLevel(getattr(logging, log_config.level.upper()))
        
        # Handler para console com cores
        console_handler = logging.StreamHandler(sys.stdout)
        console_formatter = ColoredFormatter(
            fmt='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
            datefmt='%Y-%m-%d %H:%M:%S'
        )
        console_handler.setFormatter(console_formatter)
        self.logger.addHandler(console_handler)
        
        # Handler para arquivo se configurado
        if log_config.file:
            self._setup_file_handler(log_config)
        
        # Não propaga para o logger raiz
        self.logger.propagate = False
    
    def _setup_file_handler(self, log_config) -> None:
        """Configura o handler para arquivo."""
        try:
            file_path = Path(log_config.file)
            file_path.parent.mkdir(parents=True, exist_ok=True)
            
            # Handler rotativo para arquivo
            file_handler = logging.handlers.RotatingFileHandler(
                filename=file_path,
                maxBytes=log_config.max_file_size,
                backupCount=log_config.backup_count,
                encoding='utf-8'
            )
            
            file_formatter = StructuredFormatter(
                fmt='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
                datefmt='%Y-%m-%d %H:%M:%S'
            )
            file_handler.setFormatter(file_formatter)
            
            self.logger.addHandler(file_handler)
            
        except Exception as e:
            # Se não conseguir configurar o arquivo, loga o erro mas não falha
            self.logger.warning(f"Não foi possível configurar logging para arquivo: {e}")
    
    def debug(self, message: str, **kwargs) -> None:
        """Log de debug."""
        self._log_with_extra(logging.DEBUG, message, **kwargs)
    
    def info(self, message: str, **kwargs) -> None:
        """Log de informação."""
        self._log_with_extra(logging.INFO, message, **kwargs)
    
    def warning(self, message: str, **kwargs) -> None:
        """Log de aviso."""
        self._log_with_extra(logging.WARNING, message, **kwargs)
    
    def error(self, message: str, **kwargs) -> None:
        """Log de erro."""
        self._log_with_extra(logging.ERROR, message, **kwargs)
    
    def critical(self, message: str, **kwargs) -> None:
        """Log crítico."""
        self._log_with_extra(logging.CRITICAL, message, **kwargs)
    
    def _log_with_extra(self, level: int, message: str, **kwargs) -> None:
        """Log com campos extras."""
        if kwargs:
            # Cria um registro com campos extras
            extra = {'extra_fields': kwargs}
            self.logger.log(level, message, extra=extra)
        else:
            self.logger.log(level, message)
    
    def log_operation(self, operation: str, text: str, result: str, **kwargs) -> None:
        """
        Log específico para operações de limpeza de texto.
        
        Args:
            operation: Nome da operação realizada
            text: Texto de entrada
            result: Resultado da operação
            **kwargs: Campos extras para o log
        """
        extra_fields = {
            'operation': operation,
            'input_length': len(text),
            'output_length': len(result),
            'compression_ratio': len(result) / len(text) if len(text) > 0 else 0,
            **kwargs
        }
        
        self.info(f"Operação '{operation}' concluída", **extra_fields)
    
    def log_error(self, operation: str, error: Exception, text: str = "", **kwargs) -> None:
        """
        Log específico para erros.
        
        Args:
            operation: Nome da operação que falhou
            error: Exceção que ocorreu
            text: Texto que estava sendo processado (opcional)
            **kwargs: Campos extras para o log
        """
        extra_fields = {
            'operation': operation,
            'error_type': type(error).__name__,
            'error_message': str(error),
            'input_length': len(text) if text else 0,
            **kwargs
        }
        
        self.error(f"Erro durante '{operation}': {error}", **extra_fields)
    
    def log_performance(self, operation: str, duration: float, **kwargs) -> None:
        """
        Log específico para métricas de performance.
        
        Args:
            operation: Nome da operação
            duration: Duração em segundos
            **kwargs: Campos extras para o log
        """
        extra_fields = {
            'operation': operation,
            'duration_seconds': duration,
            'duration_ms': duration * 1000,
            **kwargs
        }
        
        if duration > 1.0:
            self.warning(f"Operação '{operation}' demorou {duration:.2f}s", **extra_fields)
        else:
            self.debug(f"Operação '{operation}' concluída em {duration:.3f}s", **extra_fields)


# Logger global
logger = TextCleanerLogger()


def get_logger(name: str = None) -> TextCleanerLogger:
    """
    Retorna um logger configurado.
    
    Args:
        name: Nome do logger (opcional)
        
    Returns:
        Logger configurado
    """
    if name:
        return TextCleanerLogger(name)
    return logger



"""
Configurações centralizadas para o text_cleaner_for_py.

Este módulo define todas as configurações padrão e permite
personalização através de variáveis de ambiente ou arquivos de configuração.
"""

import os
from typing import Dict, List, Optional, Union
from dataclasses import dataclass, field

from .exceptions import ConfigurationError


@dataclass
class CleanerConfig:
    """Configurações para o processamento de texto."""
    
    # Configurações de limpeza
    remove_html: bool = True
    remove_accents: bool = True
    remove_special_chars: bool = True
    remove_extra_spaces: bool = True
    remove_emojis: bool = False
    remove_urls: bool = False
    remove_emails: bool = False
    
    # Configurações de normalização
    normalize_dates: bool = False
    normalize_numbers: bool = False
    normalize_measurements: bool = False
    normalize_proper_names: bool = False
    
    # Configurações de processamento
    default_case: str = "lower"
    supported_cases: List[str] = field(default_factory=lambda: [
        "lower", "upper", "title", "snake", "camel", "pascal"
    ])
    
    # Configurações de idioma
    default_language: str = "pt"
    supported_languages: List[str] = field(default_factory=lambda: [
        "pt", "en", "es", "fr", "de", "it"
    ])
    
    # Configurações de cache
    enable_cache: bool = True
    cache_ttl: int = 3600  # segundos
    max_cache_size: int = 1000
    
    # Configurações de performance
    max_workers: int = 4
    chunk_size: int = 1000
    enable_gpu: bool = False
    
    def validate(self) -> None:
        """Valida as configurações."""
        if self.default_case not in self.supported_cases:
            raise ConfigurationError(
                "default_case", 
                self.default_case, 
                f"Deve ser um dos valores suportados: {', '.join(self.supported_cases)}"
            )
        
        if self.default_language not in self.supported_languages:
            raise ConfigurationError(
                "default_language", 
                self.default_language, 
                f"Deve ser um dos idiomas suportados: {', '.join(self.supported_languages)}"
            )
        
        if self.max_workers < 1:
            raise ConfigurationError(
                "max_workers", 
                self.max_workers, 
                "Deve ser maior que 0"
            )
        
        if self.chunk_size < 1:
            raise ConfigurationError(
                "chunk_size", 
                self.chunk_size, 
                "Deve ser maior que 0"
            )


@dataclass
class RedisConfig:
    """Configurações para conexão com Redis."""
    
    host: str = "localhost"
    port: int = 6379
    db: int = 0
    password: Optional[str] = None
    ssl: bool = False
    timeout: int = 5
    
    def get_connection_params(self) -> Dict[str, Union[str, int, bool]]:
        """Retorna parâmetros de conexão para o Redis."""
        params = {
            "host": self.host,
            "port": self.port,
            "db": self.db,
            "ssl": self.ssl,
            "socket_timeout": self.timeout,
            "socket_connect_timeout": self.timeout
        }
        
        if self.password:
            params["password"] = self.password
            
        return params


@dataclass
class LoggingConfig:
    """Configurações para logging."""
    
    level: str = "INFO"
    format: str = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    file: Optional[str] = None
    max_file_size: int = 10 * 1024 * 1024  # 10MB
    backup_count: int = 5
    
    def validate(self) -> None:
        """Valida as configurações de logging."""
        valid_levels = ["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]
        if self.level.upper() not in valid_levels:
            raise ConfigurationError(
                "level", 
                self.level, 
                f"Deve ser um dos níveis válidos: {', '.join(valid_levels)}"
            )


class ConfigManager:
    """Gerenciador de configurações centralizado."""
    
    def __init__(self) -> None:
        """Inicializa o gerenciador de configurações."""
        self.cleaner = CleanerConfig()
        self.redis = RedisConfig()
        self.logging = LoggingConfig()
        
        self._load_from_environment()
        self._validate_all()
    
    def _load_from_environment(self) -> None:
        """Carrega configurações das variáveis de ambiente."""
        # Configurações do cleaner
        if os.getenv("TEXT_CLEANER_REMOVE_HTML"):
            self.cleaner.remove_html = os.getenv("TEXT_CLEANER_REMOVE_HTML").lower() == "true"
        
        if os.getenv("TEXT_CLEANER_DEFAULT_CASE"):
            self.cleaner.default_case = os.getenv("TEXT_CLEANER_DEFAULT_CASE")
        
        if os.getenv("TEXT_CLEANER_DEFAULT_LANGUAGE"):
            self.cleaner.default_language = os.getenv("TEXT_CLEANER_DEFAULT_LANGUAGE")
        
        if os.getenv("TEXT_CLEANER_MAX_WORKERS"):
            self.cleaner.max_workers = int(os.getenv("TEXT_CLEANER_MAX_WORKERS"))
        
        # Configurações do Redis
        if os.getenv("TEXT_CLEANER_REDIS_HOST"):
            self.redis.host = os.getenv("TEXT_CLEANER_REDIS_HOST")
        
        if os.getenv("TEXT_CLEANER_REDIS_PORT"):
            self.redis.port = int(os.getenv("TEXT_CLEANER_REDIS_PORT"))
        
        if os.getenv("TEXT_CLEANER_REDIS_PASSWORD"):
            self.redis.password = os.getenv("TEXT_CLEANER_REDIS_PASSWORD")
        
        # Configurações de logging
        if os.getenv("TEXT_CLEANER_LOG_LEVEL"):
            self.logging.level = os.getenv("TEXT_CLEANER_LOG_LEVEL")
    
    def _validate_all(self) -> None:
        """Valida todas as configurações."""
        self.cleaner.validate()
        self.logging.validate()
    
    def get_cleaner_config(self) -> CleanerConfig:
        """Retorna as configurações do cleaner."""
        return self.cleaner
    
    def get_redis_config(self) -> RedisConfig:
        """Retorna as configurações do Redis."""
        return self.redis
    
    def get_logging_config(self) -> LoggingConfig:
        """Retorna as configurações de logging."""
        return self.logging
    
    def update_cleaner_config(self, **kwargs) -> None:
        """Atualiza configurações do cleaner."""
        for key, value in kwargs.items():
            if hasattr(self.cleaner, key):
                setattr(self.cleaner, key, value)
            else:
                raise ConfigurationError(
                    key, 
                    value, 
                    f"Configuração '{key}' não existe no CleanerConfig"
                )
        
        self.cleaner.validate()
    
    def to_dict(self) -> Dict[str, Dict]:
        """Converte todas as configurações para dicionário."""
        return {
            "cleaner": self.cleaner.__dict__,
            "redis": self.redis.__dict__,
            "logging": self.logging.__dict__
        }


# Instância global de configuração
config = ConfigManager()



from typing import Optional, List, Dict, Any
import os
from pathlib import Path
import PyPDF2
from docx import Document
import re

class DocumentProcessor:
    def __init__(self):
        """Inicializa o processador de documentos."""
        self.supported_extensions = {'.pdf', '.docx', '.txt'}
        
    def read_document(self, file_path: str) -> str:
        """
        Lê o conteúdo de um documento.
        
        Args:
            file_path (str): Caminho do arquivo
            
        Returns:
            str: Conteúdo do documento
            
        Raises:
            ValueError: Se o formato do arquivo não for suportado
            FileNotFoundError: Se o arquivo não existir
        """
        file_path = Path(file_path)
        
        if not file_path.exists():
            raise FileNotFoundError(f"Arquivo não encontrado: {file_path}")
            
        if file_path.suffix not in self.supported_extensions:
            raise ValueError(f"Formato não suportado: {file_path.suffix}")
            
        if file_path.suffix == '.pdf':
            return self._read_pdf(file_path)
        elif file_path.suffix == '.docx':
            return self._read_docx(file_path)
        else:  # .txt
            return self._read_txt(file_path)
            
    def _read_pdf(self, file_path: Path) -> str:
        """Lê o conteúdo de um arquivo PDF."""
        text = []
        with open(file_path, 'rb') as file:
            pdf_reader = PyPDF2.PdfReader(file)
            for page in pdf_reader.pages:
                text.append(page.extract_text())
        return '\n'.join(text)
    
    def _read_docx(self, file_path: Path) -> str:
        """Lê o conteúdo de um arquivo DOCX."""
        doc = Document(file_path)
        return '\n'.join([paragraph.text for paragraph in doc.paragraphs])
    
    def _read_txt(self, file_path: Path) -> str:
        """Lê o conteúdo de um arquivo de texto."""
        with open(file_path, 'r', encoding='utf-8') as file:
            return file.read()
            
    def extract_metadata(self, file_path: str) -> Dict[str, Any]:
        """
        Extrai metadados do documento.
        
        Args:
            file_path (str): Caminho do arquivo
            
        Returns:
            Dict[str, Any]: Metadados do documento
        """
        file_path = Path(file_path)
        metadata = {
            'filename': file_path.name,
            'extension': file_path.suffix,
            'size': os.path.getsize(file_path),
            'created': os.path.getctime(file_path),
            'modified': os.path.getmtime(file_path)
        }
        
        if file_path.suffix == '.pdf':
            with open(file_path, 'rb') as file:
                pdf_reader = PyPDF2.PdfReader(file)
                metadata.update({
                    'pages': len(pdf_reader.pages),
                    'author': pdf_reader.metadata.get('/Author', ''),
                    'title': pdf_reader.metadata.get('/Title', ''),
                    'subject': pdf_reader.metadata.get('/Subject', '')
                })
        elif file_path.suffix == '.docx':
            doc = Document(file_path)
            metadata.update({
                'paragraphs': len(doc.paragraphs),
                'tables': len(doc.tables),
                'sections': len(doc.sections)
            })
            
        return metadata
        
    def extract_tables(self, file_path: str) -> List[List[List[str]]]:
        """
        Extrai tabelas do documento.
        
        Args:
            file_path (str): Caminho do arquivo
            
        Returns:
            List[List[List[str]]]: Lista de tabelas extraídas
        """
        file_path = Path(file_path)
        tables = []
        
        if file_path.suffix == '.docx':
            doc = Document(file_path)
            for table in doc.tables:
                table_data = []
                for row in table.rows:
                    table_data.append([cell.text for cell in row.cells])
                tables.append(table_data)
                
        return tables
        
    def extract_images(self, file_path: str, output_dir: Optional[str] = None) -> List[str]:
        """
        Extrai imagens do documento.
        
        Args:
            file_path (str): Caminho do arquivo
            output_dir (Optional[str]): Diretório para salvar as imagens
            
        Returns:
            List[str]: Lista de caminhos das imagens extraídas
        """
        # TODO: Implementar extração de imagens
        return [] 


"""
Exceções customizadas para o text_cleaner_for_py.

Este módulo define exceções específicas para diferentes tipos de erros
que podem ocorrer durante o processamento de texto.
"""

from typing import Any, Optional


class TextCleanerError(Exception):
    """Exceção base para todos os erros do text_cleaner_for_py."""
    
    def __init__(self, message: str, details: Optional[Any] = None) -> None:
        """
        Inicializa a exceção.
        
        Args:
            message: Mensagem de erro
            details: Detalhes adicionais sobre o erro
        """
        super().__init__(message)
        self.message = message
        self.details = details
    
    def __str__(self) -> str:
        """Retorna a representação string da exceção."""
        if self.details:
            return f"{self.message} (Detalhes: {self.details})"
        return self.message


class ValidationError(TextCleanerError):
    """Exceção para erros de validação de entrada."""
    
    def __init__(self, field: str, value: Any, expected_type: str) -> None:
        """
        Inicializa a exceção de validação.
        
        Args:
            field: Nome do campo com erro
            value: Valor inválido
            expected_type: Tipo esperado
        """
        message = f"Campo '{field}' deve ser do tipo {expected_type}, recebeu {type(value).__name__}: {value}"
        super().__init__(message, {"field": field, "value": value, "expected_type": expected_type})


class ConfigurationError(TextCleanerError):
    """Exceção para erros de configuração."""
    
    def __init__(self, config_key: str, config_value: Any, reason: str) -> None:
        """
        Inicializa a exceção de configuração.
        
        Args:
            config_key: Chave de configuração com erro
            config_value: Valor da configuração
            reason: Motivo do erro
        """
        message = f"Configuração inválida para '{config_key}': {config_value}. {reason}"
        super().__init__(message, {"config_key": config_key, "config_value": config_value, "reason": reason})


class ProcessingError(TextCleanerError):
    """Exceção para erros durante o processamento de texto."""
    
    def __init__(self, operation: str, text: str, reason: str) -> None:
        """
        Inicializa a exceção de processamento.
        
        Args:
            operation: Operação que falhou
            text: Texto que estava sendo processado
            reason: Motivo da falha
        """
        message = f"Erro durante '{operation}': {reason}"
        super().__init__(message, {"operation": operation, "text": text, "reason": reason})


class UnsupportedFormatError(TextCleanerError):
    """Exceção para formatos não suportados."""
    
    def __init__(self, format_name: str, supported_formats: list[str]) -> None:
        """
        Inicializa a exceção de formato não suportado.
        
        Args:
            format_name: Formato não suportado
            supported_formats: Lista de formatos suportados
        """
        message = f"Formato '{format_name}' não é suportado. Formatos suportados: {', '.join(supported_formats)}"
        super().__init__(message, {"format_name": format_name, "supported_formats": supported_formats})


class LanguageNotSupportedError(TextCleanerError):
    """Exceção para idiomas não suportados."""
    
    def __init__(self, language: str, supported_languages: list[str]) -> None:
        """
        Inicializa a exceção de idioma não suportado.
        
        Args:
            language: Idioma não suportado
            supported_languages: Lista de idiomas suportados
        """
        message = f"Idioma '{language}' não é suportado. Idiomas suportados: {', '.join(supported_languages)}"
        super().__init__(message, {"language": language, "supported_languages": supported_languages})


class FileProcessingError(TextCleanerError):
    """Exceção para erros no processamento de arquivos."""
    
    def __init__(self, file_path: str, operation: str, reason: str) -> None:
        """
        Inicializa a exceção de processamento de arquivo.
        
        Args:
            file_path: Caminho do arquivo
            operation: Operação que falhou
            reason: Motivo da falha
        """
        message = f"Erro ao processar arquivo '{file_path}' durante '{operation}': {reason}"
        super().__init__(message, {"file_path": file_path, "operation": operation, "reason": reason})


class CacheError(TextCleanerError):
    """Exceção para erros relacionados ao cache."""
    
    def __init__(self, operation: str, reason: str) -> None:
        """
        Inicializa a exceção de cache.
        
        Args:
            operation: Operação de cache que falhou
            reason: Motivo da falha
        """
        message = f"Erro no cache durante '{operation}': {reason}"
        super().__init__(message, {"operation": operation, "reason": reason})


class PerformanceError(TextCleanerError):
    """Exceção para erros de performance."""
    
    def __init__(self, operation: str, reason: str, performance_metrics: Optional[dict] = None) -> None:
        """
        Inicializa a exceção de performance.
        
        Args:
            operation: Operação que falhou
            reason: Motivo da falha
            performance_metrics: Métricas de performance (opcional)
        """
        message = f"Erro de performance durante '{operation}': {reason}"
        super().__init__(message, {"operation": operation, "reason": reason, "performance_metrics": performance_metrics})



import pytest
import asyncio
from text_cleaner_for_py.performance_cleaner import PerformanceTextCleaner

@pytest.fixture
def cleaner():
    return PerformanceTextCleaner()

def test_parallel_text_processing(cleaner):
    texts = [
        "Olá, mundo! 🧹✨",
        "Bem-vindo ao Text Cleaner!",
        "Teste de performance"
    ]
    cleaned_texts = cleaner.clean_texts_parallel(texts)
    assert len(cleaned_texts) == 3
    assert all(isinstance(text, str) for text in cleaned_texts)
    assert "🧹" not in cleaned_texts[0]

def test_large_text_processing(cleaner):
    large_text = "Olá, mundo! " * 1000
    cleaned_text = cleaner.clean_large_text(large_text, chunk_size=1000)
    assert isinstance(cleaned_text, str)
    assert len(cleaned_text) > 0

def test_text_caching(cleaner):
    text = "Olá, mundo! 🧹✨"
    result1 = cleaner.clean_text_cached(text)
    result2 = cleaner.clean_text_cached(text)
    assert result1 == result2

def test_ocr_noise_removal(cleaner):
    text = "H3ll0 W0rld! Th1s 1s 4 t3st."
    cleaned = cleaner.remove_ocr_noise(text)
    assert "H3ll0" not in cleaned
    assert "Hello" in cleaned

def test_measurement_normalization(cleaner):
    text = "O produto pesa 1.5kg e mede 2.5m"
    normalized = cleaner.normalize_measurements(text)
    assert "1.5kg" not in normalized
    assert "1.5 quilogramas" in normalized
    assert "2.5m" not in normalized
    assert "2.5 metros" in normalized

def test_duplicate_content_removal(cleaner):
    text = "Olá mundo! Olá mundo! Como vai você?"
    cleaned = cleaner.remove_duplicates(text)
    # Verifica se não há duplicatas consecutivas
    sentences = cleaned.split('.')
    sentences = [s.strip() for s in sentences if s.strip()]
    assert len(sentences) == 2  # Deve ter apenas duas frases únicas

def test_proper_name_normalization(cleaner):
    text = "joão da silva e maria santos"
    normalized = cleaner.normalize_proper_names(text)
    # Verifica se os nomes foram capitalizados corretamente
    assert "João da Silva" in normalized
    assert "Maria Santos" in normalized
    # Verifica se as preposições foram mantidas em minúsculo
    assert "da" in normalized.lower()
    assert "e" in normalized.lower()

@pytest.mark.asyncio
async def test_async_text_processing(cleaner):
    texts = [
        "Olá, mundo! 🧹✨",
        "Bem-vindo ao Text Cleaner!",
        "Teste de performance"
    ]
    cleaned_texts = await cleaner.clean_texts_async(texts)
    assert len(cleaned_texts) == 3
    assert all(isinstance(text, str) for text in cleaned_texts)

def test_gpu_processing(cleaner):
    if not cleaner.is_gpu_available():
        pytest.skip("GPU not available")
    
    large_text = "Olá, mundo! " * 10000
    cleaned_text = cleaner.clean_text_gpu(large_text)
    assert isinstance(cleaned_text, str)
    assert len(cleaned_text) > 0

def test_distributed_cache(cleaner):
    if not cleaner.is_redis_available():
        pytest.skip("Redis not available")
    
    text = "Olá, mundo! 🧹✨"
    result1 = cleaner.clean_text_distributed_cache(text)
    result2 = cleaner.clean_text_distributed_cache(text)
    assert result1 == result2

def test_clean_text_with_options(cleaner):
    text = "Olá! 👋 Visite https://exemplo.com"
    options = {
        'remove_emojis': True,
        'remove_urls': True
    }
    cleaned_text = cleaner.clean_text_with_options(text, options)
    assert "👋" not in cleaned_text
    assert "https://exemplo.com" not in cleaned_text

def test_cache_invalidation(cleaner):
    text = "Teste de cache"
    result1 = cleaner.clean_text_cached(text)
    # Força invalidação do cache
    cleaner.clean_text_cached.cache_clear()
    result2 = cleaner.clean_text_cached(text)
    assert result1 == result2

def test_parallel_processing_with_empty_list(cleaner):
    texts = []
    cleaned_texts = cleaner.clean_texts_parallel(texts)
    assert len(cleaned_texts) == 0

def test_clean_large_text_with_small_chunk_size(cleaner):
    large_text = "Olá, mundo! " * 1000
    cleaned_text = cleaner.clean_large_text(large_text, chunk_size=10)
    assert isinstance(cleaned_text, str)
    assert len(cleaned_text) > 0 


from setuptools import find_packages, setup

setup(
    name="text_cleaner_for_py",
    version="1.5.0",
    packages=find_packages(),
    install_requires=[
        "pyspellchecker>=0.7.2",
        "PyPDF2>=3.0.0",
        "python-docx>=0.8.11",
        "nltk>=3.8.1",
        "emoji>=2.8.0",
        "beautifulsoup4>=4.12.2",
        "redis>=5.0.1",
        "torch>=2.0.0",
        "numpy>=1.24.0",
        "pytest>=7.4.0",
        "pytest-asyncio>=0.21.1",
        "pytest-cov>=4.1.0",
        "pytest-benchmark>=4.0.0",
        "pytest-mock>=3.11.1",
        "pytest-timeout>=2.1.0",
        "pytest-xdist>=3.3.1",
        "pytest-sugar>=0.9.7",
        "pytest-html>=4.1.1",
        "pytest-metadata>=3.0.0",
        "pytest-env>=1.1.1",
        "pytest-randomly>=3.12.0",
        "pytest-repeat>=0.9.1",
        "pytest-rerunfailures>=12.0",
    ],
    author="Roberto Lima",
    author_email="robertolima@example.com",
    description="​💬​ Uma biblioteca Python para limpeza e processamento de texto",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/seu-usuario/text_cleaner",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: Text Processing :: General",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    python_requires=">=3.8",
    include_package_data=True,
    license="MIT",
    project_urls={
        "Homepage": "https://github.com/robertolima-dev/text-cleaner-for-py", # noqa501
        "Repository": "https://github.com/robertolima-dev/text-cleaner-for-py", # noqa501
        "Issues": "https://github.com/robertolima-dev/text-cleaner-for-py/issues", # noqa501
    },
)
